@@ -8,6 +8,7 @@ from typing import Dict, Union, Optional
 
 import rdflib
 from pydantic import HttpUrl, FileUrl, BaseModel, ConfigDict
+from rdflib.plugins.shared.jsonld.context import Context
 
 from .decorator import urirefs, namespaces, URIRefManager, NamespaceManager, _is_http_url
 from .typing import BlankNodeType
@@ -21,42 +22,27 @@ EXTRA = 'allow'  # or 'forbid' or 'ignore'
 class ThingModel(BaseModel, abc.ABC):
     """Abstract base model class to be used by model classes used within ontolutils"""
 
-    model_config = ConfigDict(extra=EXTRA)
+    model_config = ConfigDict(extra=EXTRA, populate_by_name=True)
 
     @abc.abstractmethod
     def _repr_html_(self) -> str:
         """Returns the HTML representation of the class"""
 
 
-def resolve_iri(iri: str, context: Dict) -> Union[str, None]:
-    """Resolves short iri string which uses prefix, e.g.
-     converts 'foaf:firstName' to 'http://xmlns.com/foaf/0.1/firstName'
-
-     .. note::
-
-        This function will return 'http://www.w3.org/2000/01/rdf-schema#label' if the input iri is 'label'
-        and the context does not contain 'label'.
-        If the iri could not be resolved, None is returned.
-
-
-     """
-    _iri = context.get(iri, None)
-    if _iri is not None:
-        if isinstance(_iri, dict):
-            iri = _iri.get('@id', iri)
-        else:
-            iri = _iri
-    if iri.startswith('http'):
-        return iri
-    if ':' not in iri:
-        if iri == 'label':
+def resolve_iri(key_or_iri: str, context: Context) -> Optional[str]:
+    """Resolve a key or IRI to a full IRI using the context."""
+    if key_or_iri.startswith('http'):
+        return str(key_or_iri)
+    if ':' in key_or_iri:
+        iri = context.resolve(key_or_iri)
+        if iri.startswith('http'):
+            return iri
+    try:
+        return context.terms.get(key_or_iri).id
+    except AttributeError:
+        if key_or_iri == 'label':
             return 'http://www.w3.org/2000/01/rdf-schema#label'
-        return None
-    ns, key = split_URIRef(iri)
-    prefix_iri = context.get(ns, None)
-    if prefix_iri is None:
-        return None  # could not resolve the iri
-    return f'{prefix_iri}{key}'
+    return
 
 
 @namespaces(owl='http://www.w3.org/2002/07/owl#',
@@ -121,10 +107,21 @@ class Thing(ThingModel):
             return False
         return self.id <= other.id
 
+    def pop_blank_node_id(self) -> Union[None, str]:
+        """Remove the ID if it is blank node"""
+        if self.id is None:
+            return
+        if self.id.startswith('http'):
+            return
+        _id = self.id
+        self.id = None
+        return _id
+
     def get_jsonld_dict(self,
                         context: Optional[Union[Dict, str]] = None,
                         exclude_none: bool = True,
-                        resolve_keys: bool = False) -> Dict:
+                        resolve_keys: bool = False,
+                        assign_bnode: bool = True) -> Dict:
         """Return the JSON-LD dictionary of the object. This will include the context
         and the fields of the object.
 
@@ -138,6 +135,8 @@ class Thing(ThingModel):
         resolve_keys: bool=False
             If True, then attributes of a Thing class will be resolved to the full IRI and
             explained in the context.
+        assign_bnode: bool=True
+            Assigns a blank node if no ID is set.
 
             Example:
 
@@ -172,6 +171,7 @@ class Thing(ThingModel):
                 >>>     "foaf:firstName": "John"
                 >>> }
 
+
         Returns
         -------
         Dict
@@ -190,11 +190,13 @@ class Thing(ThingModel):
 
         at_context.update(**context)
 
+        # ctx = Context(source={**at_context, **URIRefManager.get(self.__class__)})
+
         logger.debug(f'The context is "{at_context}".')
 
         def _serialize_fields(
                 obj: Union[ThingModel, int, str, float, bool, datetime],
-                exclude_none: bool = True
+                _exclude_none: bool
         ) -> Union[Dict, int, str, float, bool]:
             """Serializes the fields of a Thing object into a json-ld
             dictionary (without context!). Note, that IDs can automatically be
@@ -206,7 +208,7 @@ class Thing(ThingModel):
                 The object to serialize (a subclass of ThingModel). All other types will
                 be returned as is. One exception is datetime, which will be serialized
                 to an ISO string.
-            exclude_none: bool=True
+            _exclude_none: bool=True
                 If True, fields with None values will be excluded from the
                 serialization
 
@@ -223,24 +225,32 @@ class Thing(ThingModel):
             uri_ref_manager = URIRefManager.get(obj.__class__, None)
             at_context.update(NamespaceManager.get(obj.__class__, {}))
 
+            obj_ctx = Context(source={**context,
+                                      **NamespaceManager.get(obj.__class__, {}),
+                                      **URIRefManager.get(obj.__class__, {})})
+
             if uri_ref_manager is None:
                 return str(obj)
 
             try:
                 serialized_fields = {}
-                for k in obj.model_dump(exclude_none=exclude_none):
+                for k in obj.model_dump(exclude_none=_exclude_none):
                     value = getattr(obj, k)
                     if value is not None and k not in ('id', '@id'):
                         iri = uri_ref_manager.get(k, k)
                         if _is_http_url(iri):
                             serialized_fields[iri] = value
 
-                        ns, key = split_URIRef(iri)
-                        if key != k and not resolve_keys:
-                            at_context[k] = resolve_iri(iri, at_context)
-                            serialized_fields[k] = value
-                        else:
+                        if resolve_keys:
                             serialized_fields[iri] = value
+                        else:
+                            term = obj_ctx.find_term(obj_ctx.expand(iri))
+                            if term:
+                                if obj_ctx.shrink_iri(term.id).split(':')[1] != k:
+                                    at_context[k] = term.id
+                                    serialized_fields[k] = value
+                                else:
+                                    serialized_fields[iri] = value
             except AttributeError as e:
                 raise AttributeError(f"Could not serialize {obj} ({obj.__class__}). Orig. err: {e}") from e
 
@@ -249,13 +259,14 @@ class Thing(ThingModel):
                 _field = serialized_fields.pop(k)
                 key = k
                 if isinstance(v, Thing):
-                    serialized_fields[key] = _serialize_fields(v, exclude_none=exclude_none)
+                    serialized_fields[key] = _serialize_fields(v, _exclude_none=_exclude_none)
                 elif isinstance(v, list):
-                    serialized_fields[key] = [_serialize_fields(i, exclude_none=exclude_none) for i in v]
+                    serialized_fields[key] = [
+                        _serialize_fields(i, _exclude_none=_exclude_none) for i in v]
                 elif isinstance(v, (int, float)):
                     serialized_fields[key] = v
                 else:
-                    serialized_fields[key] = _serialize_fields(v)
+                    serialized_fields[key] = _serialize_fields(v, _exclude_none=_exclude_none)
 
             _type = URIRefManager[obj.__class__].get(obj.__class__.__name__, obj.__class__.__name__)
 
@@ -264,10 +275,11 @@ class Thing(ThingModel):
             if obj.id is not None:
                 out["@id"] = obj.id
             else:
-                out["@id"] = rdflib.BNode()
+                if assign_bnode:
+                    out["@id"] = rdflib.BNode().n3()
             return out
 
-        serialization = _serialize_fields(self, exclude_none=exclude_none)
+        serialization = _serialize_fields(self, _exclude_none=exclude_none)
 
         jsonld = {
             "@context": at_context,
@@ -279,7 +291,8 @@ class Thing(ThingModel):
                           context: Optional[Dict] = None,
                           exclude_none: bool = True,
                           rdflib_serialize: bool = False,
-                          resolve_keys: bool = True) -> str:
+                          resolve_keys: bool = True,
+                          assign_bnode: bool = True) -> str:
         """Similar to model_dump_json() but will return a JSON string with
         context resulting in a JSON-LD serialization. Using `rdflib_serialize=True`
         will use the rdflib to serialize. This will make the output a bit cleaner
@@ -302,6 +315,8 @@ class Thing(ThingModel):
         resolve_keys: bool=False
             If True, then attributes of a Thing class will be resolved to the full IRI and
             explained in the context.
+        assign_bnode: bool=True
+            Assigns a blank node if no ID is set.
 
             .. seealso:: `Thing.get_jsonld_dict`
 
@@ -313,7 +328,8 @@ class Thing(ThingModel):
         jsonld_dict = self.get_jsonld_dict(
             context=context,
             exclude_none=exclude_none,
-            resolve_keys=resolve_keys
+            resolve_keys=resolve_keys,
+            assign_bnode=assign_bnode
         )
         jsonld_str = json.dumps(jsonld_dict, indent=4)
         if not rdflib_serialize:
@@ -331,23 +347,21 @@ class Thing(ThingModel):
                            context=_context,
                            indent=4)
 
-    def __repr__(self):
+    def __repr__(self, limit: Optional[int] = None):
         _fields = {k: getattr(self, k) for k in self.model_fields if getattr(self, k) is not None}
-        repr_fields = ", ".join([f"{k}={v}" for k, v in _fields.items()])
-        if self.model_config['extra'] == 'allow':
-            if len(self.model_extra) > 0:
-                repr_extra = ", ".join([f"{k}={v}" for k, v in self.model_extra.items()])
-                return f"{self.__class__.__name__}({repr_fields}, {repr_extra})"
-        return f"{self.__class__.__name__}({repr_fields})"
+        repr_extra = ", ".join([f"{k}={v}" for k, v in {**_fields, **self.model_extra}.items()])
+        if limit is None or len(repr_extra) < limit:
+            return f"{self.__class__.__name__}({repr_extra})"
+        return f"{self.__class__.__name__}({repr_extra[0:limit]}...)"
 
-    def __str__(self):
-        return self.__repr__()
+    def __str__(self, limit: Optional[int] = None):
+        return self.__repr__(limit=limit)
 
     def _repr_html_(self) -> str:
         """Returns the HTML representation of the class"""
-        _fields = {k: getattr(self, k) for k in self.model_fields if getattr(self, k) is not None}
-        repr_fields = ", ".join([f"{k}={v}" for k, v in _fields.items()])
-        return f"{self.__class__.__name__}({repr_fields})"
+        # _fields = {k: getattr(self, k) for k in self.model_fields if getattr(self, k) is not None}
+        # repr_fields = ", ".join([f"{k}={v}" for k, v in _fields.items()])
+        return self.__repr__()
 
     @classmethod
     def from_jsonld(cls,
