@@ -2,10 +2,10 @@ import datetime
 import json
 import logging
 import unittest
-
+from rdflib.plugins.shared.jsonld.context import Context
 import pydantic
 import rdflib
-from pydantic import EmailStr
+from pydantic import EmailStr, Field
 
 from ontolutils import Thing, urirefs, namespaces, get_urirefs, get_namespaces
 from ontolutils import set_logging_level
@@ -31,18 +31,51 @@ class TestNamespaces(unittest.TestCase):
         set_logging_level(self.INITIAL_LOG_LEVEL)
         assert logging.getLogger('ontolutils').level == self.INITIAL_LOG_LEVEL
 
+    def test_model_fields(self):
+        @namespaces(foaf="http://xmlns.com/foaf/0.1/")
+        @urirefs(Agent='foaf:Agent',
+                 name='foaf:lastName',
+                 age='foaf:age')
+        class Agent(Thing):
+            """Pydantic Model for http://xmlns.com/foaf/0.1/Agent
+            Parameters
+            ----------
+            mbox: EmailStr = None
+                Email address (foaf:mbox)
+            """
+            name: str = Field(default=None, alias="lastName")  # name is synonymous to lastName
+            age: int = None
+
+        agent = Agent(name='John Doe', age=23)
+        self.assertEqual(agent.name, 'John Doe')
+        self.assertEqual(agent.age, 23)
+
+        # extra fields are allowed and either the model field or its uriref can be used
+        agent = Agent(lastName='Doe', age=23)
+        self.assertEqual(agent.name, 'Doe')
+        self.assertEqual(agent.age, 23)
+
+        agent = Agent(age=23)
+        self.assertEqual(agent.name, None)
+        self.assertEqual(agent.age, 23)
+
     def test_resolve_iri(self):
-        ret = resolve_iri('foaf:age', context={'foaf': 'http://xmlns.com/foaf/0.1/'})
+        ret = resolve_iri('foaf:age', context=Context(source={'foaf': 'http://xmlns.com/foaf/0.1/'}))
         self.assertEqual(ret, 'http://xmlns.com/foaf/0.1/age')
-        ret = resolve_iri('age', context={'foaf': 'http://xmlns.com/foaf/0.1/'})
+
+        ret = resolve_iri('age', context=Context(source={'foaf': 'http://xmlns.com/foaf/0.1/'}))
         self.assertEqual(ret, None)
-        ret = resolve_iri('age', context={'age': 'http://xmlns.com/foaf/0.1/age'})
+
+        ret = resolve_iri('age', context=Context(source={'age': 'http://xmlns.com/foaf/0.1/age'}))
         self.assertEqual(ret, 'http://xmlns.com/foaf/0.1/age')
-        ret = resolve_iri('label', context={'age': 'http://xmlns.com/foaf/0.1/age'})
+
+        ret = resolve_iri('label', context=Context(source={'age': 'http://xmlns.com/foaf/0.1/age'}))
         self.assertEqual(ret, 'http://www.w3.org/2000/01/rdf-schema#label')
-        ret = resolve_iri('label', context={'label': {'@id': 'http://www.w3.org/2000/01/rdf-schema#label'}})
+
+        ret = resolve_iri('label', context=Context(source={'label': {'@id': 'http://www.w3.org/2000/01/rdf-schema#label'}}))
         self.assertEqual(ret, 'http://www.w3.org/2000/01/rdf-schema#label')
-        ret = resolve_iri('prefix:label', {})
+
+        ret = resolve_iri('prefix:label', Context(source={}))
         self.assertEqual(ret, None)
 
     def test_split_URIRef(self):
@@ -60,6 +93,12 @@ class TestNamespaces(unittest.TestCase):
     def test_thing_custom_prop(self):
         """It is helpful to have the properties equal to the urirefs keys,
         however, this should not be required!"""
+        @namespaces(foaf='http://xmlns.com/foaf/0.1/',
+                    schema='http://www.schema.org/')
+        @urirefs(Affiliation='prov:Affiliation',
+                 name='schema:name')
+        class Affiliation(Thing):
+            name: str
 
         @namespaces(foaf='http://xmlns.com/foaf/0.1/',
                     prov='http://www.w3.org/ns/prov#')
@@ -68,12 +107,17 @@ class TestNamespaces(unittest.TestCase):
                  lastName='foaf:lastName',
                  age='foaf:age')
         class Person(Thing):
-            first_name: str = None
+            first_name: str = Field(default=None, alias='firstName')
             lastName: str
             age: int = None
+            affiliation: Affiliation=None
 
         p = Person(first_name='John', lastName='Doe', age=23)
-        print(p.model_dump_jsonld(resolve_keys=True))
+        person_json = p.model_dump_jsonld(resolve_keys=False)
+        self.assertEqual(json.loads(person_json)['first_name'], 'John')
+        person_json = p.model_dump_jsonld(resolve_keys=True)
+        self.assertEqual(json.loads(person_json)['foaf:firstName'], 'John')
+
         p_from_jsonld = Person.from_jsonld(data=p.model_dump_jsonld(resolve_keys=True), limit=1)
         self.assertEqual(p_from_jsonld.first_name, 'John')
         self.assertEqual(p_from_jsonld.lastName, 'Doe')
@@ -125,7 +169,7 @@ class TestNamespaces(unittest.TestCase):
         with self.assertRaises(TypeError):
             thing.get_jsonld_dict(context=1)
 
-        thing_dict = thing.get_jsonld_dict()
+        thing_dict = thing.get_jsonld_dict(resolve_keys=True)
         self.assertIsInstance(thing_dict, dict)
         self.assertDictEqual(thing_dict['@context'],
                              {'owl': 'http://www.w3.org/2002/07/owl#',
@@ -165,10 +209,11 @@ class TestNamespaces(unittest.TestCase):
             agent.model_validate(agent.model_dump())
         agent.mbox = 'my@email.com'
         jsonld_str1 = agent.model_dump_jsonld(rdflib_serialize=False)
+        self.assertTrue('@id' in json.loads(jsonld_str1))
 
         self.assertIsInstance(json.loads(jsonld_str1)['foaf:age'], int)
 
-        jsonld_str2 = agent.model_dump_jsonld(rdflib_serialize=True)
+        jsonld_str2 = agent.model_dump_jsonld(rdflib_serialize=True)  # will assign blank node! Pop it later
         jsonld_str2_dict = json.loads(jsonld_str2)
         self.assertNotEqual(
             json.loads(jsonld_str1),
@@ -220,6 +265,7 @@ class TestNamespaces(unittest.TestCase):
             label='Agent 1',
             mbox='my@email.com'
         )
+        self.assertEqual(agent.id, None)
         ns = agent.namespaces
         jsonld_string = agent.model_dump_jsonld(
             context={
@@ -232,6 +278,14 @@ class TestNamespaces(unittest.TestCase):
         self.assertDictEqual(loaded_agent.namespaces, ns)
         self.assertEqual(loaded_agent.mbox, agent.mbox)
 
+        # do the same with thing:
+        thing = Thing.from_jsonld(data=jsonld_string, limit=1)
+        self.assertEqual(thing.label, 'Agent 1')
+        self.assertTrue(thing.id.startswith('_:'))
+        _id = thing.id
+        self.assertEqual(thing.pop_blank_node_id(), _id)
+        self.assertEqual(thing.id, None)
+
     def test_schema_http(self):
         @namespaces(foaf="http://xmlns.com/foaf/0.1/",
                     schema="https://schema.org/")
@@ -279,7 +333,14 @@ class TestNamespaces(unittest.TestCase):
                 label='Organization 1'
             ),
         )
-        jsonld_str = person.model_dump_jsonld()
+        jsonld_str = person.model_dump_jsonld(resolve_keys=True)
+        jsonld_dict = json.loads(jsonld_str)
+
+        self.assertEqual(jsonld_dict['schema:affiliation']['@type'], 'prov:Organization')
+        self.assertEqual(jsonld_dict['schema:affiliation']['rdfs:label'], 'Organization 1')
+        self.assertEqual(jsonld_dict['rdfs:label'], 'Person 1')
+        self.assertEqual(jsonld_dict['@type'], 'foaf:Person')
+
 
     def test_prov(self):
         @namespaces(prov="https://www.w3.org/ns/prov#",
@@ -369,26 +430,12 @@ class TestNamespaces(unittest.TestCase):
             "foaf:firstName": "John",
             "foaf:lastName": "Doe"
         }
-        jsonld_dict = json.loads(mt.model_dump_jsonld(resolve_keys=True))
-        jsonld_dict.pop('@id')
+        jsonld_dict = json.loads(mt.model_dump_jsonld())
+        jsonld_dict.pop('@id', None)
         self.assertDictEqual(jsonld_dict,
                              ref_jsonld)
 
-        ref_jsonld = {
-            "@context": {
-                "owl": "http://www.w3.org/2002/07/owl#",
-                "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
-                "foaf": "http://xmlns.com/foaf/0.1/",
-                "first_name": "http://xmlns.com/foaf/0.1/firstName",
-                "last_name": "http://xmlns.com/foaf/0.1/lastName"
-            },
-            "@type": "CustomPerson",
-            "first_name": "John",
-            "last_name": "Doe"
-        }
-        jsonld_dict = json.loads(mt.model_dump_jsonld(resolve_keys=False))
-        jsonld_dict.pop('@id')
+        jsonld_dict = json.loads(mt.model_dump_jsonld())
+        jsonld_dict.pop('@id', None)
         self.assertDictEqual(jsonld_dict,
                              ref_jsonld)
-        from pprint import pprint
-        pprint(jsonld_dict)
